/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */
import { ChannelType, type Client } from "discord.js";
import { channels } from "../config/channels.js";
import { logger } from "../utils/logger.js";

/**
 * Posts a daily standup reminder in configured channels.
 * @param client - The Discord client.
 */
export const standup = async(client: Client): Promise<void> => {
  try {
    const mapped = await Promise.all(
      channels.map(async(channel) => {
        const fetched = await client.channels.fetch(channel).catch(() => {
          void logger.log("warn", `Failed to fetch channel ${channel}.`);
          return null;
        });
        if (!fetched) {
          await logger.log("warn", `Channel ${channel} not found.`);
          return null;
        }
        if (fetched.type !== ChannelType.GuildText) {
          await logger.log("warn", `Channel ${channel} is not a text channel.`);
          return null;
        }
        return fetched;
      }),
    );
    const filtered = mapped.filter((channel) => {
      return channel !== null;
    });
    await Promise.all(
      filtered.map(async(channel) => {
        await channel.send(
          // eslint-disable-next-line stylistic/max-len -- This message is too long to fit on one line.
          "Good morning @everyone~! It's time for standup! Please share your goals for today.",
        ).catch(() => {
          void logger.log("warn", `Failed to send standup reminder in channel ${channel.name}.`);
        });
      }),
    );
  } catch (error) {
    if (error instanceof Error) {
      await logger.error("standup module", error);
      return;
    }
    await logger.error("standup module", new Error(String(error)));
  }
};
